@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/common/logger.py
 ##
-## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2017 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2004-2005 Vincent Hanquez <tab AT snarc.org>
 ## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
@@ -579,8 +579,7 @@ class Logger:
                 message_col, subject_col)
         return self.commit_to_db(values, write_unread)
 
-    def get_last_conversation_lines(self, jid, restore_how_many_rows,
-                    pending_how_many, timeout, account):
+    def get_last_conversation_lines(self, jid, pending_how_many, account):
         """
         Accept how many rows to restore and when to time them out (in minutes)
         (mark them as too old) and number of messages that are in queue and are
@@ -595,8 +594,15 @@ class Logger:
             return []
         where_sql, jid_tuple = self._build_contact_where(account, jid)
 
+        # How many lines to restore and when to time them out
+        restore_how_many = gajim.config.get('restore_lines')
+        if restore_how_many <= 0:
+            return
+        timeout = gajim.config.get('restore_timeout')  # in minutes
+
         now = int(float(time.time()))
-        timed_out = now - (timeout * 60) # before that they are too old
+        if timeout != 0:
+            timeout = now - (timeout * 60) # before that they are too old
         # so if we ask last 5 lines and we have 2 pending we get
         # 3 - 8 (we avoid the last 2 lines but we still return 5 asked)
         try:
@@ -606,8 +612,8 @@ class Logger:
                 ORDER BY time DESC LIMIT %d OFFSET %d
                 ''' % (where_sql, constants.KIND_SINGLE_MSG_RECV,
                 constants.KIND_CHAT_MSG_RECV, constants.KIND_SINGLE_MSG_SENT,
-                constants.KIND_CHAT_MSG_SENT, constants.KIND_ERROR, timed_out,
-                restore_how_many_rows, pending_how_many), jid_tuple)
+                constants.KIND_CHAT_MSG_SENT, constants.KIND_ERROR, timeout,
+                restore_how_many, pending_how_many), jid_tuple)
 
             results = self.cur.fetchall()
         except sqlite.DatabaseError:
