@@ -2,7 +2,7 @@
 ## src/common/optparser.py
 ##
 ## Copyright (C) 2003-2005 Vincent Hanquez <tab AT snarc.org>
-## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2017 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
 ##                         Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
@@ -27,7 +27,6 @@
 ##
 
 import os
-import sys
 import locale
 import re
 from time import time
@@ -241,6 +240,8 @@ class OptionsParser:
             self.update_config_to_01601()
         if old < [0, 16, 4, 1] and new >= [0, 16, 4, 1]:
             self.update_config_to_01641()
+        if old < [0, 16, 6, 1] and new >= [0, 16, 6, 1]:
+            self.update_config_to_01661()
 
         gajim.logger.init_vars()
         gajim.logger.attach_cache_database()
@@ -946,3 +947,10 @@ class OptionsParser:
             gajim.config.set_per('accounts', account, 'connection_types',
                 ' '.join(connection_types))
         gajim.config.set('version', '0.16.4.1')
+
+    def update_config_to_01661(self):
+        for account in self.old_values['accounts'].keys():
+            resource = self.old_values['accounts'][account]['resource']
+            if resource.lower() == 'gajim':
+                gajim.config.set_per('accounts', account, 'resource', 'gajim.$rand')
+        gajim.config.set('version', '0.16.6.1')
